@@ -11,6 +11,7 @@ pub mod env;
 pub mod files;
 pub mod hashes;
 pub mod lock;
+pub mod manifest;
 pub mod outputs;
 pub mod run;
 pub mod tasks;
@@ -161,11 +162,11 @@ fn main() -> Result<()> {
                     .get(&task_name)
                     .ok_or(FasterError::TargetNotFound(task_name))?;
 
-                if task.files.is_empty() {
+                if task.inputs.is_empty() {
                     return Ok(());
                 }
 
-                for file in GlobFilesReader::get_paths_from_patterns(&workdir, &task.files)? {
+                for file in GlobFilesReader::get_paths_from_patterns(&workdir, &task.inputs)? {
                     println!("{}", file.to_string_lossy());
                 }
 
@@ -181,7 +182,7 @@ fn main() -> Result<()> {
                     .get(&task_name)
                     .ok_or(FasterError::TargetNotFound(task_name))?;
 
-                for file in GlobFilesReader::get_paths_from_patterns(&workdir, &task.files)? {
+                for file in GlobFilesReader::get_paths_from_patterns(&workdir, &task.inputs)? {
                     all_paths.push(file.to_string_lossy().to_string());
                 }
             }
