@@ -1,78 +1,70 @@
 use anyhow::{Context, Result};
-use std::{fs::File, path::Path};
+use std::fs::File;
 
-use crate::{files::FilesReader, tasks::Task, util, FasterError};
+use crate::{files::FilesReader, tasks::TaskContext, util, FasterError};
 
-pub fn save_outputs<R: FilesReader, P1, P2>(
-    workdir: P1,
-    task_name: &str,
-    task_path: P2,
+pub fn archive_outputs<R: FilesReader>(
+    ctx: &TaskContext,
     current_hash: u64,
-    task: &Task,
-) -> Result<()>
-where
-    P1: AsRef<Path>,
-    P2: AsRef<Path>,
-{
-    if task.outputs.is_empty() {
-        return Ok(());
+) -> Result<Vec<String>> {
+    if ctx.task.outputs.is_empty() {
+        return Ok(vec![]);
     }
 
-    let paths = R::get_paths_from_patterns(&workdir, &task.outputs)?;
+    let paths = R::get_paths_from_patterns(&ctx.workdir, &ctx.task.outputs)?;
 
-    if paths.is_empty() && !task.ignore_missing_outputs {
+    if paths.is_empty() && !ctx.task.ignore_missing_outputs {
         return Err(FasterError::NoOutputsFound {
-            task: task_name.to_owned(),
+            task: ctx.task_name.to_owned(),
         }
         .into());
     }
 
-    let archive_path =
-        workdir
-            .as_ref()
-            .join(task_path)
-            .join(format!("{}.{}", current_hash, util::ARCHIVE_SUFFIX));
+    let archive_path = ctx.workdir.join(ctx.task_path.as_path()).join(format!(
+        "{}.{}",
+        current_hash,
+        util::ARCHIVE_SUFFIX
+    ));
 
-    let archive_file = File::create(archive_path)
-        .with_context(|| format!("failed to create output archive for task `{}`", task_name))?;
+    let archive_file = File::create(archive_path).with_context(|| {
+        format!(
+            "failed to create output archive for task `{}`",
+            ctx.task_name
+        )
+    })?;
     let compressed_writer = snap::write::FrameEncoder::new(archive_file);
     let mut archive = tar::Builder::new(compressed_writer);
 
+    let mut results = Vec::with_capacity(paths.len());
     for path in &paths {
         // All patterns we receive here are absolute, so we need to trim
         // the prefix to get the path in the current workdir
-        let path_in_workdir = path.strip_prefix(&workdir)?;
+        let path_in_workdir = path.strip_prefix(&ctx.workdir)?;
 
         archive
             .append_path_with_name(path, path_in_workdir)
             .with_context(|| format!("failed to append path {:?} to archive", path))?;
+
+        results.push(String::from(path.to_string_lossy()));
     }
 
-    Ok(archive.finish()?)
+    archive.finish()?;
+    Ok(results)
 }
 
-pub fn restore_outputs<P1, P2>(
-    workdir: P1,
-    task_path: P2,
-    current_hash: u64,
-    task: &Task,
-) -> Result<()>
-where
-    P1: AsRef<Path>,
-    P2: AsRef<Path>,
-{
+pub fn restore_outputs(ctx: &TaskContext, hash: u64) -> Result<()> {
     // Adding/removing outputs will invalidate the hash,
     // which means we can assume there were no defined outputs
     // if there are no outputs in the task config.
-    if task.outputs.is_empty() {
+    if ctx.task.outputs.is_empty() {
         return Ok(());
     }
 
-    let archive_path =
-        workdir
-            .as_ref()
-            .join(task_path)
-            .join(format!("{}.{}", current_hash, util::ARCHIVE_SUFFIX));
+    let archive_path = ctx.workdir.join(ctx.task_path.as_path()).join(format!(
+        "{}.{}",
+        hash,
+        util::ARCHIVE_SUFFIX
+    ));
 
     if !archive_path.is_file() {
         return Ok(());
@@ -84,16 +76,21 @@ where
 
     // Since the output patterns are always relative to the current directory,
     // we can safely unpack cached artifacts to the current directory as well
-    Ok(archive.unpack(workdir)?)
+    Ok(archive.unpack(&ctx.workdir)?)
 }
 
 #[cfg(test)]
 mod tests {
-    use std::{fs::OpenOptions, path::PathBuf};
+    use std::{
+        fs::OpenOptions,
+        path::{Path, PathBuf},
+    };
 
     use super::*;
     use crate::{
+        env::ResolvedEnv,
         files::GlobFilesReader,
+        tasks::Task,
         util::{ensure_task_file_path, tests::run_in_temp_dir},
     };
 
@@ -140,10 +137,17 @@ mod tests {
         let task_path =
             ensure_task_file_path(&workdir, task_name).expect("failed to ensure task path");
 
-        save_outputs::<GlobFilesReader, _, _>(&workdir, task_name, &task_path, 0u64, &task)
-            .expect("failed to save outputs");
+        let ctx = TaskContext {
+            workdir: PathBuf::from(workdir.as_ref()),
+            task_name: task_name.to_owned(),
+            task,
+            task_path,
+            task_env: ResolvedEnv::new(),
+        };
 
-        let mut archive_path = PathBuf::from(&task_path);
+        archive_outputs::<GlobFilesReader>(&ctx, 0u64).expect("failed to save outputs");
+
+        let mut archive_path = PathBuf::from(&ctx.task_path);
         archive_path.push("0.tar.sz");
         assert!(archive_path.is_file());
 
@@ -160,8 +164,7 @@ mod tests {
             );
         }
 
-        restore_outputs(&workdir.as_ref(), &task_path.as_path(), 0u64, &task)
-            .expect("failed to restore outputs");
+        restore_outputs(&ctx, 0u64).expect("failed to restore outputs");
 
         // Files should now be restored
         for file in &files {
