@@ -5,18 +5,19 @@ use indicatif::{MultiProgress, ProgressBar, ProgressDrawTarget, ProgressStyle};
 use lazy_static::lazy_static;
 use std::{
     collections::HashSet,
-    fs::{self, File},
-    io::Write,
+    fs,
     path::{Path, PathBuf},
     thread::{self, JoinHandle},
-    time::{Instant, SystemTime},
+    time::Instant,
 };
 
 use crate::{
     env::{resolve_env, ResolvedEnv},
     files::GlobFilesReader,
-    hashes, lock, outputs,
-    tasks::{Task, TaskGraph},
+    hashes, lock,
+    manifest::{store_and_clean_results, Manifest},
+    outputs::restore_outputs,
+    tasks::{Task, TaskContext, TaskGraph},
     util, Config, FasterError,
 };
 
@@ -31,13 +32,17 @@ lazy_static! {
         ProgressStyle::with_template("  {prefix} {wide_msg}").expect("failed to create done style");
 }
 
+const FASTER_ROOT_VAR: &str = "FASTER_ROOT";
+const FASTER_TASK_VAR: &str = "FASTER_TASK";
+
 struct TaskMessage(String, Task);
 
-#[derive(Clone)]
 enum CommandResult {
     Success(String),
     Failed(String),
+    CachedFailed(String),
     Cached,
+    Empty,
     Cancelled,
     Skipped,
 }
@@ -233,12 +238,29 @@ fn run_worker<P>(
 
                 should_abort = true;
             }
+            Ok(CommandResult::CachedFailed(_output)) => {
+                bar.set_style(DONE_STYLE.clone());
+                bar.set_prefix(format!("{}", style("✗").red()));
+                bar.finish_with_message(format!(
+                    "{} {}",
+                    style("cached").red().bold().dim(),
+                    task_name
+                ));
+
+                should_abort = true;
+            }
             Ok(CommandResult::Cached) => {
                 bar.set_style(DONE_STYLE.clone());
                 bar.set_prefix(format!("{}", style("✔").green()));
 
                 bar.finish_with_message(format!("{} {}", style("cached").black().dim(), task_name));
             }
+            Ok(CommandResult::Empty) => {
+                bar.set_style(DONE_STYLE.clone());
+                bar.set_prefix(format!("{}", style("✔").green()));
+
+                bar.finish_with_message(format!("{} {}", style("empty").black().dim(), task_name));
+            }
             Err(_) => {
                 bar.set_style(DONE_STYLE.clone());
                 bar.set_prefix(format!("{}", style("✗").red()));
@@ -292,137 +314,118 @@ where
     bar.set_style(PROGRESS_STYLE.clone());
     bar.set_message(format!("{} {}", style("running").blue().dim(), task_name));
 
-    let hash_path = task_path.join(util::SUM_FILE);
-    let current_hash =
-        hashes::calc_hash::<GlobFilesReader, _>(&workdir, task, global_env, &task_env)?;
-
-    let has_changes = match hashes::find_existing_hash(&hash_path)? {
-        Some(previous_hash) => current_hash != previous_hash,
-        _ => true,
+    let ctx = TaskContext {
+        workdir: PathBuf::from(workdir.as_ref()),
+        task_name: task_name.to_owned(),
+        task: task.clone(),
+        task_path,
+        task_env,
     };
 
-    if !has_changes {
-        outputs::restore_outputs(&workdir, &task_path, current_hash, task)?;
-        return Ok(CommandResult::Cached);
+    let mut manifest = Manifest::load(&ctx)?;
+
+    // We're about to either restore old outputs or new create new inputs
+    // by running the defined script. This means that we should first look
+    // if we had previous outputs and remove them.
+    if let Some(previous_outputs) = manifest
+        .get_latest_run()
+        .and_then(|run| run.outputs.as_ref())
+    {
+        // These were previously added to the archive, so they are guaranteed to
+        // be files. Just try to delete them 1 by 1 and ignore failures.
+        for path in previous_outputs {
+            let _ = fs::remove_file(path);
+        }
     }
 
+    let current_hash = hashes::calc_hash::<GlobFilesReader>(global_env, &ctx)?;
+    if let Some(stored_run) = manifest.runs.iter().find(|run| run.sum == current_hash) {
+        if stored_run.success {
+            restore_outputs(&ctx, stored_run.sum)?;
+            return Ok(CommandResult::Cached);
+        }
+
+        // The last run was found, and failed. Retrieve the logs and
+        // return them again.
+        return Ok(CommandResult::CachedFailed(String::new()));
+    }
+
+    if task.script.is_none() {
+        return Ok(CommandResult::Empty);
+    }
+
+    // Script is guaranteed to be Some here
+    let script = task.script.as_ref().unwrap();
+
     // Run the actual task
-    if let Some(script) = &task.script {
-        // TODO: Make the run command configurable
-        let args = vec!["-c", script];
-        let mut command = duct::cmd("sh", &args)
-            .stderr_to_stdout()
-            .stdout_capture()
-            .unchecked();
+    // TODO: Make the run command configurable
+    let args = vec!["-exo", "pipefail", "-c", script];
+    let mut command = duct::cmd("bash", &args)
+        .stderr_to_stdout()
+        .stdout_capture()
+        .unchecked();
 
-        for (key, value) in global_env {
-            command = command.env(key, value);
+    command = command
+        .env(FASTER_ROOT_VAR, workdir.as_ref().as_os_str())
+        .env(FASTER_TASK_VAR, &ctx.task_name);
+
+    for (key, value) in global_env {
+        command = command.env(key, value);
+    }
+
+    for (key, value) in &ctx.task_env {
+        command = command.env(key, value);
+    }
+
+    let mut last_instant = Instant::now();
+    let handle = command.start()?;
+
+    loop {
+        if let Ok(()) = abort_rx.try_recv() {
+            // We don't really care about the child here,
+            // even if it doesn't stop we don't want to block
+            let _ = handle.kill();
+            return Ok(CommandResult::Cancelled);
         }
 
-        for (key, value) in task_env {
-            command = command.env(key, value);
-        }
+        match handle.try_wait() {
+            Ok(Some(result)) => {
+                // TODO: Can we do this without cloning stdout? Might use up a lot
+                // of memory when there's a lot of output from the command
+                // NOTE: stdout and sterr are merged into stdout, so we're
+                // getting both in the correct order by reading stdout here.
+                let logs = String::from_utf8(result.stdout.clone())?;
 
-        let mut last_instant = Instant::now();
-        let handle = command.start()?;
+                store_and_clean_results(
+                    &ctx,
+                    &mut manifest,
+                    result.status.success(),
+                    result.status.code(),
+                    current_hash,
+                    &logs,
+                )?;
 
-        loop {
-            if let Ok(()) = abort_rx.try_recv() {
-                // We don't really care about the child here,
-                // even if it doesn't stop we don't want to block
-                let _ = handle.kill();
-                return Ok(CommandResult::Cancelled);
+                return if result.status.success() {
+                    Ok(CommandResult::Success(logs))
+                } else {
+                    Ok(CommandResult::Failed(logs))
+                };
+            }
+            Err(err) => {
+                return Err(err.into());
             }
 
-            match handle.try_wait() {
-                Ok(Some(result)) => {
-                    // TODO: Can we do this without cloning stdout? Might use up a lot
-                    // of memory when there's a lot of output from the command
-                    let logs = String::from_utf8(result.stdout.clone())?;
-                    save_and_clean_logs(&task_path, current_hash, &logs)?;
-
-                    if !result.status.success() {
-                        return Ok(CommandResult::Failed(logs));
-                    }
-
-                    // TODO: Run clean command/clean outputs
-
-                    // Only save hash and outputs if the script succeeded
-                    hashes::save_hash(&hash_path, current_hash)?;
-                    outputs::save_outputs::<GlobFilesReader, _, _>(
-                        workdir,
-                        task_name,
-                        &task_path,
-                        current_hash,
-                        task,
-                    )?;
-
-                    return Ok(CommandResult::Success(logs));
-                }
-                Err(err) => {
-                    return Err(err.into());
+            Ok(None) => {
+                // TODO: Add a configurable command timeout?
+                if last_instant.elapsed().as_millis() < 100 {
+                    continue;
                 }
 
-                Ok(None) => {
-                    // TODO: Add a configurable command timeout?
-                    if last_instant.elapsed().as_millis() < 100 {
-                        continue;
-                    }
-
-                    bar.inc(1);
-                    last_instant = Instant::now();
-                }
+                bar.inc(1);
+                last_instant = Instant::now();
             }
         }
     }
-
-    unreachable!();
-}
-
-fn save_and_clean_logs(task_path: &Path, current_hash: u64, output: &str) -> Result<()> {
-    let mut log_path = PathBuf::from(task_path);
-    log_path.push(format!("{}.{}", current_hash, util::LOG_SUFFIX));
-
-    let mut file = File::create(log_path.clone())?;
-    file.write_all(output.as_bytes())?;
-
-    // Now, stat all files in the directory and delete all except the n-1 newest
-    // (since we keep the newest one by default)
-    let mut log_files = fs::read_dir(task_path)?
-        // Just filter out invalid entries here
-        .filter_map(|entry| entry.ok())
-        .filter_map(|entry| entry.metadata().ok().map(|metadata| (entry, metadata)))
-        .filter_map(|(entry, metadata)| {
-            entry
-                .path()
-                .extension()
-                .map(|ext| (entry, metadata, ext.to_string_lossy().into_owned()))
-        })
-        .filter(|(entry, metadata, ext)| {
-            // Filter out the newest hash
-            entry.path() != log_path &&
-            // Filter out directories
-            metadata.is_file() &&
-            // Filter out non-logfiles
-            ext.as_str() == util::LOG_SUFFIX
-        })
-        .map(|(entry, metadata, _)| (entry, metadata))
-        .collect::<Vec<_>>();
-
-    // Sort from oldest to newest
-    log_files.sort_by_key(|(_, metadata)| metadata.created().unwrap_or(SystemTime::UNIX_EPOCH));
-    log_files.reverse();
-
-    log_files
-        .iter()
-        .skip(util::KEEP_LAST_RESULTS - 1)
-        .for_each(|(entry, _)| {
-            // TODO: Log the error in verbose mode here, this is
-            // not a blocker though
-            let _ = std::fs::remove_file(entry.path());
-        });
-
-    Ok(())
 }
 
 #[cfg(test)]
@@ -459,6 +462,34 @@ mod tests {
         assert!(matches!(result, Ok(())));
     }
 
+    #[test]
+    fn run_empty() {
+        let workdir = run_in_temp_dir();
+
+        let config: crate::Config = serde_yaml::from_str(
+            r#"
+            tasks:
+              build: {}
+            "#,
+        )
+        .unwrap();
+
+        let mut task_graph =
+            TaskGraph::single(&config, "build").expect("failed to build task graph");
+
+        let (_, rx) = crossbeam_channel::unbounded::<()>();
+        let result = run(
+            &workdir,
+            &config,
+            &mut task_graph,
+            indicatif::ProgressDrawTarget::hidden(),
+            1,
+            rx,
+        );
+
+        assert!(matches!(result, Ok(())));
+    }
+
     #[test]
     fn run_multi() {
         let workdir = run_in_temp_dir();
@@ -496,4 +527,27 @@ mod tests {
 
         assert!(matches!(result, Ok(())));
     }
+
+    #[test]
+    fn exit_code_failed() {
+        let workdir = run_in_temp_dir();
+
+        let config: crate::Config =
+            serde_yaml::from_str(r#"tasks: {build: {script: "exit 1"}}"#).unwrap();
+
+        let mut task_graph =
+            TaskGraph::build(&config, "build").expect("failed to build task graph");
+
+        let (_, rx) = crossbeam_channel::unbounded::<()>();
+        let result = run(
+            &workdir,
+            &config,
+            &mut task_graph,
+            indicatif::ProgressDrawTarget::hidden(),
+            1,
+            rx,
+        );
+
+        assert!(matches!(result, Ok(())));
+    }
 }
