@@ -1,29 +1,23 @@
 use anyhow::Result;
 use petgraph::{stable_graph::StableDiGraph, EdgeDirection};
 use serde::Deserialize;
-use std::collections::{hash_map, HashMap, VecDeque};
+use std::{
+    collections::{hash_map, HashMap, VecDeque},
+    path::PathBuf,
+};
 
-use crate::{Config, FasterError};
+use crate::{env::ResolvedEnv, Config, FasterError};
 
 type NodeIndex = petgraph::graph::NodeIndex<petgraph::stable_graph::DefaultIx>;
 
 #[derive(Debug, Clone, Deserialize, Default)]
+#[serde(default, rename_all = "camelCase")]
 pub struct Task {
     pub script: Option<String>,
-
-    #[serde(default)]
     pub environment: HashMap<String, String>,
-
-    #[serde(default)]
     pub dependencies: Vec<String>,
-
-    #[serde(default)]
-    pub files: Vec<String>,
-
-    #[serde(default)]
+    pub inputs: Vec<String>,
     pub outputs: Vec<String>,
-
-    #[serde(default, rename = "ignoreMissingOutputs")]
     pub ignore_missing_outputs: bool,
 }
 
@@ -33,6 +27,15 @@ pub struct TaskGraph<'a> {
     indices: HashMap<&'a str, NodeIndex>,
 }
 
+#[derive(Debug, Default)]
+pub struct TaskContext {
+    pub workdir: PathBuf,
+    pub task_name: String,
+    pub task: Task,
+    pub task_path: PathBuf,
+    pub task_env: ResolvedEnv,
+}
+
 impl<'a> TaskGraph<'a> {
     pub fn task_count(&self) -> usize {
         self.graph.node_count()
